%{



#include "config.h"

#include <stdlib.h>


#include "macro.h"
#include "constant.h"
#include "opaque.h"
#include "oprs-type.h"
#include "parser-funct.h"

#include "oprs-type_f.h"





#include "parser-funct_f.h"


#include "oprs-server-parser.tab.h"






#include "shashPack_f.h"







#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER->yy_is_interactive ) \
		{ \
		int c = '*', n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else if ( ((result = fread( buf, 1, max_size, yyin )) == 0) \
		  && (parse_source == PS_FILE) && ferror( yyin ) ) \
		YY_FATAL_ERROR( "input in flex scanner failed" );


%}





ws		[ \t]+
real 		[-\+]?[0-9]+\.[0-9]*
exp		[eE][-\+]?[0-9]+
integer 	[-\+]?[0-9]+[l]?
long_long 	[-\+]?[0-9]+ll
pointer 	0x[0123456789abcdefABCDEF]+
comment		;.*$
qstring 	\"([^"]|\\["])*\" 
untypedid	([^ \t\n~0123456789"&\(\)\$@;\|\.[\]][^ \t\n"\(\)\]]*)
typedid 	([^ \t\n~0123456789:"&\(\)\$@;\|\.[\]][^ \t\n:"\(\)\]]*)

idbar   	(\|[^|]+\|)
nl		\n
or		V
and		&
op		\(
cp		\)
opar		\[
cpar		\]
oplisp		\(\.
cplisp		\.\)
lvar		\${id}
pvar		@{id}
gpvar		@@{id}


id {untypedid}

     
%s COMMAND

%%
{ws}		;
<COMMAND>h	{return HELP;}
<COMMAND>help	{return HELP;}
<COMMAND>\?	{return HELP;}

<COMMAND>show		{return SHOW; }
<COMMAND>version	{return VERSION; }
<COMMAND>copyright	{return COPYRIGHT; }
<COMMAND>add		{return ADD_TK; }
<COMMAND>send	 	{return SEND; }
<COMMAND>broadcast 	{return BROADCAST; }
<COMMAND>make		{return MAKE_OPRS; }
<COMMAND>make_x		{return MAKE_X_OPRS; }
<COMMAND>accept		{return ACCEPT_OPRS; }
<COMMAND>connect	{return CONNECT; }
<COMMAND>kill		{return KILL_OPRS; }
<COMMAND>rehash		{return REHASH_TK; }
<COMMAND>reset		{return RESET_TK; }
<COMMAND>parser		{return PARSER_TK; }
<COMMAND>kernel		{return KERNEL_TK; }
<COMMAND>transmit	{return TRANSMIT; }
<COMMAND>transmit_all	{return TRANSMIT_ALL; }
<COMMAND>quit		{return QUIT; }
<COMMAND>q		{return QUIT; }
<COMMAND>exit		{return QUIT; }
<COMMAND>include	{return INCLUDE; }





^\.\n		{ parser_line_number[parser_index]++;
                  return RESET_DOT_TK;} 


{real}		|
{real}{exp}    	{ sscanf(oprs_yytext,"%lf",&oprs_yylval.real);
		  return REAL_TK;}
{pointer}       { sscanf(oprs_yytext+2,"%x", (unsigned int *)&oprs_yylval.pointer); 
		  return POINTER_TK;}
{long_long}       { sscanf(oprs_yytext,"%lld", &oprs_yylval.long_long);
		  return LONG_LONG_TK;}
{integer}       { sscanf(oprs_yytext,"%d", &oprs_yylval.integer);
		  return INTEGER_TK;}
{comment}	;
{qstring}	{ NEWQSTR(oprs_yytext,oprs_yylval.string);
		  return QSTRING_TK;}

\|\|		{ oprs_yylval.string = or_sym;
		  return SYMBOL_TK;}
&		{ oprs_yylval.string = and_sym;
		  return SYMBOL_TK;}
~		{ oprs_yylval.string = not_sym;
		  return SYMBOL_TK;}


{lvar}		{ DECLARE_VAR(oprs_yytext,oprs_yylval.string);
		  return LOGICAL_VAR_TK;}
{pvar} |
{gpvar}		{ DECLARE_VAR(oprs_yytext,oprs_yylval.string);
		  return PROGRAM_VAR_TK;}


\+>		{  oprs_yylval.string = cond_add_sym;
		  return SYMBOL_TK;}
->		{  oprs_yylval.string = cond_del_sym;
		  return SYMBOL_TK;}
=>	|

[Aa][Ss][Ss][Ee][Rr][Tt]	|
[Cc][Oo][Nn][Cc][Ll][Uu][Dd][Ee]	{  oprs_yylval.string = temp_conclude_sym;
		  return SYMBOL_TK;}
~>	|
[Rr][Ee][Tt][Rr][Aa][Cc][Tt] 		{  oprs_yylval.string = retract_sym;
		  return SYMBOL_TK;}
!	|
[Aa][Cc][Hh][Ii][Ee][Vv][Ee] 		{  oprs_yylval.string = achieve_sym;
		  return SYMBOL_TK;}
\?	|
[Tt][Ee][Ss][Tt]	 		{  oprs_yylval.string = test_sym;
		  return SYMBOL_TK;}
#	|
[Pp][Rr][Ee][Ss][Ee][Rr][Vv][Ee]	{  oprs_yylval.string = preserve_sym;
		  return SYMBOL_TK;}
\^	|
[Ww][Aa][Ii][Tt]	 		{  oprs_yylval.string = twait_sym;
		  return SYMBOL_TK;}
\%	|
[Mm][Aa][Ii][Nn][Tt][Aa][Ii][Nn]	{  oprs_yylval.string = maintain_sym;
		  return SYMBOL_TK;}
\<>	|
[Cc][Aa][Ll][Ll]			{  oprs_yylval.string = call_sym;
		  return SYMBOL_TK;}


{oplisp}        { return OP_LISP_TK;}
{cplisp}        { return CP_LISP_TK;}
{opar}          { return OP_ARRAY_TK;}
{cpar}          { return CP_ARRAY_TK;}


t	|
T		{ oprs_yylval.string = lisp_t_sym;
		  return SYMBOL_TK;}
[nN][iI][lL]	{ oprs_yylval.string = nil_sym;
		  return SYMBOL_TK;}

{id} |
{idbar}		{ DECLARE_ID(oprs_yytext,oprs_yylval.string);
		  return SYMBOL_TK;}

{op}        	{ return OPENP_TK;}
{cp}        	{ return CLOSEP_TK;}

{nl}	    	{parser_line_number[parser_index]++;}

.		{warning("unexpected character.");}


%%
#include "parser-funct.h"
#include "yy-funct.c"

void yy_begin_0()
{
     BEGIN 0;
}

void yy_begin_COMMAND()
{
     BEGIN COMMAND;
}
